module ts7600_top(
output debug_rxd_pad,
input debug_txd_pad,

inout [7:0] nand_dat_pad,
input nand_rdn_pad,
input nand_wrn_pad,
input nand_ale_pad,
//input nand_cle_pad,
output nand_rdy_pad,
input nand_cs0n_pad, 
//input nand_cs1n_pad,

output [1:0] sd_fault_led,

output led_grnn_pad,
output led_redn_pad,

inout [66:0] dio_pad,

input fpga_reset_pad,
input fpga_12mhz_pad,

output cpu_reset_pad,
output hard_rebootn_pad,

output eth_right_led_pad,
input eth_100mbit_led_pad,

output [2:0] fpga_irq_pad,

/*input [2:0] spare_pad,
output sd_act_led_pad,
output en_usb_5v_pad,*/
output [2:0] boot_strap_pad
);

/* software currently requires these to be enabled/disabled contiguously. */
parameter xuart0_opt = 1'b1;
parameter xuart1_opt = 1'b1;
parameter xuart2_opt = 1'b1;
parameter xuart3_opt = 1'b1;
parameter xuart4_opt = 1'b1;
parameter xuart5_opt = 1'b1;
parameter xuart6_opt = 1'b1;
parameter xuart7_opt = 1'b1;
parameter spi_opt = 1'b1;

/****************************************************************************
* Boilerplate FPGA configuration (clocks, pll, reset, etc.)
*****************************************************************************/
wire wb_clk;
assign wb_clk = pll_75mhz;
reg rst; 
wire wb_rst;
assign wb_rst = rst;
reg [31:0] timer;
reg cpu_reset;
reg [2:0] mode;
assign cpu_reset_pad = cpu_reset;
assign hard_rebootn_pad = sc_reboot ? 1'b0 : 1'bz;
assign boot_strap_pad = mode;
wire [1:0] sd_led;
assign sd_fault_led = ~(sd_led);

wire lock, pll_75mhz, pll_100mhz;
pll0 clkgen (
  .CLK(fpga_12mhz_pad),
  //.CLKOP( ),
  .CLKOK(pll_75mhz),
  .CLKOK2(pll_100mhz),
  .LOCK(lock)
);

always @(posedge internal_osc_o or posedge fpga_reset_pad) begin
  if(fpga_reset_pad) begin
    cpu_reset <= 1'b1;
    mode <= 3'b000;
    timer <= 32'd0;
  end else begin
    timer <= timer + 1'b1;
    if(timer > 32'd64) cpu_reset <= 1'b0;
    if(cpu_reset) begin
      case({dio_i[5], dio_i[7]})
        2'b11: mode <= 3'b010;
        default: mode <= 3'b101;
      endcase
    end
  end
end

always @(posedge pll_75mhz or posedge fpga_reset_pad) begin
  if(fpga_reset_pad) begin
    rst <= 1'b1;
  end else begin
    rst <= 1'b0;
  end
end

assign eth_right_led_pad = dio_oe[0] ? dio_o[0] : !eth_100mbit_led_pad;
assign led_grnn_pad = !led_grn;
assign led_redn_pad = !led_red;
assign fpga_irq_pad[2] = 1'b0;

/****************************************************************************
* NBUS interface, NAND bus like interface
*****************************************************************************/
wire cpuwbm_cyc_o, cpuwbm_stb_o, cpuwbm_we_o;
reg [15:0] cpuwbm_dat_i;
wire [15:0] cpuwbm_dat_o;
wire [7:0] cpuwbm_adr_o;
reg cpuwbm_ack_i;
wire [1:0] cpuwbm_sel_o;

wire nand_dat_oe;
wire [7:0] nand_dat_o;
wire [7:0] nand_dat_i;

genvar i;
generate
  for(i = 0; i < 8; i = i+1) begin: nandbus
    BB nand_dat_bb(.I(nand_dat_o[i]), .T(!nand_dat_oe), .B(nand_dat_pad[i]), .O(nand_dat_i[i]));
  end
endgenerate
mxsnandbridge nand_bus (
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),
  .wbm_cyc_o(cpuwbm_cyc_o),
  .wbm_stb_o(cpuwbm_stb_o),
  .wbm_dat_o(cpuwbm_dat_o),
  .wbm_adr_o(cpuwbm_adr_o),
  .wbm_we_o(cpuwbm_we_o),
  .wbm_dat_i(cpuwbm_dat_i),
  .wbm_ack_i(cpuwbm_ack_i),
  .wbm_sel_o(cpuwbm_sel_o),
  
  .nand_dat_i(nand_dat_i),
  .nand_dat_o(nand_dat_o),
  .nand_dat_oe_o(nand_dat_oe),
  .nand_stbn_i(nand_wrn_pad),
  .nand_rdnwr_i(nand_rdn_pad),
  .nand_csn_i(nand_cs0n_pad),
  .nand_ale_i(nand_ale_pad),
  
	.nand_rdyn_o(nand_rdy_pad)
);

/****************************************************************************
 * TS-BLOCKRAM interface, port to wishbone and NBUS
 ****************************************************************************/
wire ramwbs1_cyc_i, ramwbs1_stb_i, ramwbs1_we_i, ramwbs1_ack_o;
wire [31:0] ramwbs1_dat_i, ramwbs1_dat_o;
wire [31:0] ramwbs1_adr_i;
wire ramwbs2_cyc_i, ramwbs2_stb_i, ramwbs2_we_i, ramwbs2_ack_o;
wire [31:0] ramwbs2_dat_i, ramwbs2_dat_o;
wire [31:0] ramwbs2_adr_i;
wire [3:0] ramwbs2_sel_i, ramwbs1_sel_i;

blockram_8kbyte sramcore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  /* To XUART */
  .wb1_cyc_i(ramwbs1_cyc_i),
  .wb1_stb_i(ramwbs1_stb_i),
  .wb1_we_i(ramwbs1_we_i),
  .wb1_dat_i(ramwbs1_dat_i),
  .wb1_dat_o(ramwbs1_dat_o),
  .wb1_adr_i(ramwbs1_adr_i),
  .wb1_ack_o(ramwbs1_ack_o),
  .wb1_sel_i(ramwbs1_sel_i),

  /* To wb_memwindow */
  .wb2_cyc_i(ramwbs2_cyc_i),
  .wb2_stb_i(ramwbs2_stb_i),
  .wb2_we_i(ramwbs2_we_i),
  .wb2_dat_i(ramwbs2_dat_i),
  .wb2_dat_o(ramwbs2_dat_o),
  .wb2_adr_i(ramwbs2_adr_i),
  .wb2_ack_o(ramwbs2_ack_o),
  .wb2_sel_i(ramwbs2_sel_i)
);

/****************************************************************************
* XUART Super UARTS
*****************************************************************************/
wire xuwbs_ack_o;
reg xuwbs_en;
wire [31:0] uartwbm_adr_o;
wire [15:0] uartwbm_dat_o, xuwbs_dat_o;
wire [15:0] uartwbm_dat_i;
reg [7:0] xuart_rx;
wire [7:0] xuart_active;
wire [7:0] xuart_cts = dio_i[48:41];
wire [7:0] xuart_txen, xuart_tx;
reg [7:0] xuart_rx_q, xuart_cts_q;
wire xuart_irq;
assign fpga_irq_pad[0] = xuart_irq;
wire [7:0] xuart_disable = {!xuart7_opt[0], !xuart6_opt[0], !xuart5_opt[0],
  !xuart4_opt[0], !xuart3_opt[0], !xuart2_opt[0], !xuart1_opt[0],
  !xuart0_opt[0]};
wire xuart_opt = (xuart_disable[7:0] != 8'hff);
always @(posedge wb_clk) begin
  xuart_rx_q <= xuart_rx;
  xuart_cts_q <= xuart_cts;
end
assign ramwbs1_dat_i[31:0] = {uartwbm_dat_o[15:0], uartwbm_dat_o[15:0]};
assign ramwbs1_adr_i = {uartwbm_adr_o[15:2], 2'b00};
assign uartwbm_dat_i[15:0] =
  uartwbm_adr_o[1] ? ramwbs1_dat_o[31:16] : ramwbs1_dat_o[15:0];
assign ramwbs1_sel_i = uartwbm_adr_o[1] ? 4'b1100 : 4'b0011;
ts_xuart tsxuartcore(
  .wb_clk_i(wb_clk & xuart_opt),
  .wb_rst_i(wb_rst),

  .wbm_cyc_o(ramwbs1_cyc_i),
  .wbm_stb_o(ramwbs1_stb_i),
  .wbm_we_o(ramwbs1_we_i),
  .wbm_ack_i(ramwbs1_ack_o),
  .wbm_adr_o(uartwbm_adr_o),
  .wbm_dat_i(uartwbm_dat_i),
  .wbm_dat_o(uartwbm_dat_o),


  .wbs_cyc_i(cpuwbm_cyc_o && xuwbs_en),
  .wbs_stb_i(cpuwbm_stb_o && xuwbs_en),
  .wbs_we_i(cpuwbm_we_o),
  .wbs_adr_i(cpuwbm_adr_o),
  .wbs_dat_i(cpuwbm_dat_o),
  .wbs_dat_o(xuwbs_dat_o),
  .wbs_ack_o(xuwbs_ack_o),

  .baudclk_i(pll_100mhz),
  .cts_i(xuart_cts_q),
  .rx_i(xuart_rx_q),
  .tx_o(xuart_tx),
  .txen_o(xuart_txen),
  .active_o(xuart_active),
  .irq_o(xuart_irq),
  .uart_disable_i(xuart_disable)
);

/****************************************************************************
 * NBUS blockram window (for XUART 8kbyte memory access)
 ****************************************************************************/
reg mwinwbs_en;
wire [15:0] mwinwbs_dat_o;
wire mwinwbs_ack_o;
wb_memwindow16to32 mwincore(
  .wb_clk_i(wb_clk && xuart_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(cpuwbm_cyc_o && mwinwbs_en),
  .wb_stb_i(cpuwbm_stb_o && mwinwbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_dat_o(mwinwbs_dat_o),
  .wb_ack_o(mwinwbs_ack_o),

  .wbm_cyc_o(ramwbs2_cyc_i),
  .wbm_stb_o(ramwbs2_stb_i),
  .wbm_we_o(ramwbs2_we_i),
  .wbm_adr_o(ramwbs2_adr_i),
  .wbm_dat_o(ramwbs2_dat_i),
  .wbm_dat_i(ramwbs2_dat_o),
  .wbm_ack_i(ramwbs2_ack_o),
  .wbm_sel_o(ramwbs2_sel_i)
);



/****************************************************************************
 * SPI controller logic
 ****************************************************************************/
reg spiwbs_en;
wire [15:0] spiwbs_dat_o;
wire spiwbs_ack_o;
wire scken;
wire [3:0] csn;
reg [3:0] csn_q;
always @(posedge wb_clk) csn_q <= csn;
//assign ser_flash_cs_padn = csn[0];
wire [3:0] si = {4{dio_i[12]}};
wire sck, hispeed, fclk_in, mosi_in;
reg flash_clk, flash_clk_oe, flash_mosi, flash_mosi_oe;
BB fclk(.I(flash_clk), .T(!flash_clk_oe), .B(flash_clk_pad), .O(fclk_in));
BB mosi(.I(flash_mosi), .T(!flash_mosi_oe), .B(flash_mosi_pad), .O(mosi_in));
wire so;
wb_spi #(
  .only8bit_opt(1'b1)
) spicore(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk & spi_opt),
  .wb_cyc_i(cpuwbm_cyc_o && spiwbs_en),
  .wb_stb_i(cpuwbm_stb_o && spiwbs_en),
  .wb_dat_o(spiwbs_dat_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_we_i(cpuwbm_we_o),
  .wb_ack_o(spiwbs_ack_o),
  .wb_sel_i(cpuwbm_sel_o),

  .scken_o(scken),
  .so_o(so),
  .si_i(si),
  .csn_o(csn),
  .sck_o(sck),
  .hispeed_o(hispeed)
);


/****************************************************************************
* DIO assignments and Syscon
*****************************************************************************/
reg scwbs_en;
wire led_grn;
wire led_red;
wire evgpio_irq;
assign fpga_irq_pad[1] = evgpio_irq;
wire [15:0] scwbs_dat_o;
wire scwbs_ack_o;
wire [66:0] dio_i;
reg [66:0] dio_o, dio_oe;
wire [66:0] dio_o_sc, dio_oe_sc;
wire [7:0] tx_en;
genvar y;
generate
  for(y = 0; y < 67; y = y+1) begin: dio
    BB dio_bb(.I(dio_o[y]), .T(!dio_oe[y]), .B(dio_pad[y]), .O(dio_i[y]));
  end
endgenerate
wire sc_reboot;
wire internal_osc_o;
integer x;
always @(*) begin
  for(x = 0; x < 67; x = x+1) begin
    dio_o[x] = dio_o_sc[x];
    dio_oe[x] = dio_oe_sc[x];
  end
	
  /* DIO7 is used as a bootstrap pin */
  if(cpu_reset) dio_oe[7] = 1'b0;

 /* SPI controller hijacks some pins when LUN#1, #2, or #3 is active */
  flash_clk = /*hispeed ? (pll_75mhz_shifted | !scken) : */sck;
  flash_mosi = so;
  flash_clk_oe = 1'b1;
  flash_mosi_oe = 1'b1;
  if (spi_opt) begin
    if (csn_q[0] == 1'b0) begin
      dio_oe[11] = 1'b1;
      dio_o[11] = 1'b1;
    end
    if (csn_q[1] == 1'b0) begin
      dio_o[37] = 1'b1;
      dio_oe[37] = 1'b1;
    end
    if (csn_q[2] == 1'b0) begin
      dio_o[39] = 1'b1;
      dio_oe[39] = 1'b1;
    end
    if (csn[0] == 1'b0) begin
      dio_o[11] = csn[0];
      dio_oe[11] = 1'b1;
    end
    if (csn[1] == 1'b0) begin
      dio_o[37] = csn[1];
      dio_oe[37] = 1'b1;
    end
    if (csn[2] == 1'b0) begin
      dio_o[39] = csn[2];
      dio_oe[39] = 1'b1;
    end
    if (csn[2:0] != 3'b111) begin
      /* Currently, hispeed mode DOES NOT WORK */
      dio_o[14] = /*hispeed ? (pll_75mhz_shifted | !scken) : */sck;
      dio_o[13] = so;
      dio_oe[14:12] = 3'b110;
    end
  end

  /* XUART TXENs share pins with SPI, TXENs will take precedence */
  xuart_rx[0] = dio_i[6];
  if (xuart_active[0]) begin
    dio_oe[5] = 1'b1;
    dio_o[5] = xuart_tx[0];
    dio_oe[6] = 1'b0;
    if(tx_en[0]) begin
      dio_o[37] = xuart_txen[0];
      dio_oe[37] = 1'b1;
    end
  end
  xuart_rx[1] = dio_i[20];
  if (xuart_active[1]) begin
    dio_oe[19] = 1'b1;
    dio_o[19] = xuart_tx[1];
    dio_oe[20] = 1'b0;
    if(tx_en[1]) begin
      dio_o[27] = xuart_txen[1];
      dio_oe[27] = 1'b1;
    end
  end
  xuart_rx[2] = dio_i[22];
  if (xuart_active[2]) begin
    dio_oe[21] = 1'b1;
    dio_o[21] = xuart_tx[2];
    dio_oe[22] = 1'b0;
    if(tx_en[2]) begin
      dio_o[28] = xuart_txen[2];
      dio_oe[28] = 1'b1;
    end
  end
  xuart_rx[3] = dio_i[24];
/*  if (can_opt && can_enable) begin
    dio_oe[23] = 1'b1;
    dio_o[23] = can_tx;
    dio_oe[24] = 1'b0;
  end else */
  if (xuart_active[3]) begin
    dio_oe[23] = 1'b1;
    dio_o[23] = xuart_tx[3];
    dio_oe[24] = 1'b0;
    if(tx_en[3]) begin
      dio_o[38] = xuart_txen[3];
      dio_oe[38] = 1'b1;
    end
  end
  xuart_rx[4] = dio_i[26];
  if (xuart_active[4]) begin
    dio_o[25] = xuart_tx[4];
    dio_oe[25] = 1'b1;
    dio_oe[26] = 1'b0;
    if(tx_en[4]) begin
      dio_o[39] = xuart_txen[4];
      dio_oe[39] = 1'b1;
    end
  end
  xuart_rx[5] = dio_i[32];
  if (xuart_active[5]) begin
    dio_oe[31] = 1'b1;
    dio_o[31] = xuart_tx[5];
    dio_oe[32] = 1'b0;
    if(tx_en[5]) begin
      dio_o[29] = xuart_txen[5];
      dio_oe[29] = 1'b1;
    end
  end
  xuart_rx[6] = dio_i[34];
  if (xuart_active[6]) begin
    dio_o[33] = xuart_tx[6];
    dio_oe[33] = 1'b1;
    dio_oe[34] = 1'b0;
    if(tx_en[6]) begin
      dio_o[30] = xuart_txen[6];
      dio_oe[30] = 1'b1;
    end
  end
  xuart_rx[7] = dio_i[36];
  if (xuart_active[7]) begin
    dio_o[35] = xuart_tx[7];
    dio_oe[35] = 1'b1;
    dio_oe[36] = 1'b0;
    if(tx_en[7]) begin
      dio_o[40] = xuart_txen[7];
      dio_oe[40] = 1'b1;
    end
  end
end


syscon mxs_syscon(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(cpuwbm_cyc_o && scwbs_en),
  .wb_stb_i(cpuwbm_stb_o && scwbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_sel_i(cpuwbm_sel_o),
  .wb_dat_o(scwbs_dat_o),
  .wb_ack_o(scwbs_ack_o),

  .dio_i(dio_i),
  .dio_oe_o(dio_oe_sc),
  .dio_o(dio_o_sc),
  .evgpio_irq_o(evgpio_irq),

  .cpu_uart_rxd_o(debug_rxd_pad),
  .cpu_uart_txd_i(debug_txd_pad),

  .internal_osc_o(internal_osc_o),
  .clk_100mhz_i(pll_100mhz),
  .reboot_o(sc_reboot),
  .led_grn_o(led_grn),
  .led_red_o(led_red),
  .mode_i(mode),
	.sd_led_o(sd_led),
  .tx_en_o(tx_en)
  //.mode3_i,
  //.pllphase_o,
  //.can_enable_o,
  //.can_wbaccess_i
);


always @(*) begin
  xuwbs_en = 1'b0;
  mwinwbs_en = 1'b0;
  scwbs_en = 1'b0;
  spiwbs_en = 1'b0;
  cpuwbm_ack_i = 1'b1;
  cpuwbm_dat_i = 16'hxxxx;

  if(cpuwbm_adr_o >= 8'h60) begin
    spiwbs_en = 1'b1;
    cpuwbm_dat_i = spiwbs_dat_o;
    cpuwbm_ack_i = spiwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h5c) begin
    mwinwbs_en = 1'b1;
    cpuwbm_dat_i = mwinwbs_dat_o;
    cpuwbm_ack_i = mwinwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h40) begin
    xuwbs_en = 1'b1;
    cpuwbm_dat_i = xuwbs_dat_o;
    cpuwbm_ack_i = xuwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h00) begin
    scwbs_en = 1'b1;
    cpuwbm_dat_i = scwbs_dat_o;
    cpuwbm_ack_i = scwbs_ack_o;
  end

end

endmodule
